#!/usr/bin/env ruby
#   A simple script for managing Redmine users from command line.
#   Run this script using runner in script directory
#
options = { :environment => (ENV['RAILS_ENV'] || "development").dup }
ENV["RAILS_ENV"] = options[:environment]
RAILS_ENV.replace(options[:environment]) if defined?(RAILS_ENV)
#
require File.expand_path('../../config/boot',  __FILE__)
require File.expand_path('../../config/environment',  __FILE__)
require 'optparse'
require 'readline'
require 'pp'
###
def user_list
  puts "%4s %-20s %-20s %-20s" % %w(ID LOGIN FIRST_NAME LAST_NAME)
  puts "-" * 64
  User.all.sort! do |a,b|
    a.id <=> b.id
  end.each do |user|
    puts "%4d %-20s %-20s %-20s" % [user.id ,user.login, user.firstname, user.lastname]
  end
end
#
def user_check(inuser=nil)
  return if inuser.nil?
  oUser = User.find_by_login(inuser)
  if oUser.nil?
    oUser = User.find_by_id(inuser)  
  end
  if oUser.nil?
    oUser = User.new
    oUser.login = inuser
    oUser.password = "dummydummy"
    oUser.firstname = inuser
    oUser.lastname = inuser
    oUser.mail = inuser + "@nowhere.com"
    oUser.admin = false
    begin 
      oUser.save!
    rescue Exception => e
      puts "Error saving user",e.message
    end
  end
end
#
def user_delete(inuser=nil)
  return if inuser.nil?
  oUser = User.find_by_login(inuser)
  oUser.delete if !oUser.nil?
end
#
def user_password(inuser=nil, inpassword=nil)
  return if inuser.nil?
  return if inpassword.nil?
  oUser = User.find_by_login(inuser)
  oUser.password = inpassword
  begin 
    oUser.save!
  rescue Exception => e
    puts "Error saving password",e.message
  end
end 
#
def user_role(inuser=nil,inrole)
  return if inuser.nil?
  oUser = User.find_by_login(inuser)
  oUser.admin = inrole
  begin 
    oUser.save!
  rescue Exception => e
    puts "Error saving admin",e.message
  end
end 
#
options = {} 
optparse = OptionParser.new do |opts|                 
  opts.on "-l","--list","List the exisitng redmine users " do
    user_list
  end
  opts.on "-u" ,"--user","=MANDATORY","Identifies user " do |inuser|
    options[:user] = inuser
    user_check(inuser)
  end
  opts.on "-d","--delete","=MANDATORY","Deletes user " do |inuser|
    user_delete(inuser)
  end
  opts.on "-p","--password","=MANDATORY","Changes password  " do |inpassword|
    user_password(options[:user],inpassword)
  end
  opts.on "--roleadmin","Sets admin mode " do
    user_role(options[:user],true)
  end
  opts.on "--roleuser","Sets user mode " do
    user_role(options[:user],false)
  end
end
begin
  optparse.parse!
rescue 
  ### TODO: this is wrong since $! is a runner 
  puts $!
  puts optparse
end